/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat_notification_sensitivity.h"
#include "config.h"

void roccat_notification_sensitivity_audio_cancel(void *data) {
	gint retval;

	if (!data)
		return;

	retval = ca_context_cancel((ca_context *)data, ROCCAT_NOTIFICATION_SENSITIVITY_ID);
	if (retval != 0 && retval != CA_ERROR_STATE)
		g_warning("Roccat audio notification: %s", ca_strerror(retval));
}

void roccat_notification_sensitivity_audio_update(gpointer data, guint x, guint y) {
	// not implemented
}

void roccat_notification_sensitivity_audio_update_up(gpointer data) {
	gint retval;

	if (!data)
		return;

	roccat_notification_sensitivity_audio_cancel(data);
	retval = ca_context_play((ca_context *)data, ROCCAT_NOTIFICATION_SENSITIVITY_ID,
			CA_PROP_MEDIA_FILENAME, INSTALL_PREFIX "/share/roccat/sounds/sensitivity/sensitivity-up.oga",
			NULL);
	if (retval)
		g_warning("Roccat audio notification: %s", ca_strerror(retval));
}

void roccat_notification_sensitivity_audio_update_down(gpointer data) {
	gint retval;

	if (!data)
		return;

	roccat_notification_sensitivity_audio_cancel(data);
	retval = ca_context_play((ca_context *)data, ROCCAT_NOTIFICATION_SENSITIVITY_ID,
			CA_PROP_MEDIA_FILENAME, INSTALL_PREFIX "/share/roccat/sounds/sensitivity/sensitivity-down.oga",
			NULL);
	if (retval)
		g_warning("Roccat audio notification: %s", ca_strerror(retval));
}

gpointer roccat_notification_sensitivity_audio_new(RoccatNotificator *notificator) {
	return notificator->audio_usable ? notificator->context : NULL;
}

void roccat_notification_sensitivity_audio_free(gpointer data) {}
